using System;
using System.Data;
using System.Data.SqlClient;
using SACI.Comum.DataSet;
using System.Configuration;

namespace SACI.AcessoDados
{

    public abstract class cBaseAcessoDados
    {
        protected SqlCommand selectCommand = null;
        protected SqlCommand insertCommand = null;
        protected SqlCommand updateCommand = null;
        protected SqlCommand deleteCommand = null;

        private SqlConnection connection = null;
        protected SqlDataAdapter dataAdapter = null;

        private DateTime initialTime = DateTime.MinValue;
        private DateTime finalTime = DateTime.MinValue;
        private int timeoutTime = 30;

        #region Properties
        private DateTime BeginProcessing
        {
            get
            {
                return initialTime;
            }
            set
            {
                initialTime = DateTime.Now;
            }
        }
        private DateTime EndProcessing
        {
            get
            {
                return finalTime;
            }
            set
            {
                finalTime = DateTime.Now;
            }
        }
        protected int TimeoutTime
        {
            get
            {
                return timeoutTime;
            }
            set
            {
                timeoutTime = value;
            }
        }
        #endregion

        #region Constructor
        public cBaseAcessoDados()
        {
            selectCommand = new SqlCommand();
            dataAdapter = new SqlDataAdapter();
//            connection = new SqlConnection("Password=saci;Persist Security Info=True;User ID=saci;Initial Catalog=saci;Data Source=localhost\\sqlexpress");
//            connection = new SqlConnection("Password=as;Persist Security Info=True;User ID=sa;Initial Catalog=saci_2;Data Source=seginf01");


#if DEBUG
            System.Diagnostics.Trace.WriteLine("=======================================================");
            System.Diagnostics.Trace.WriteLine("ConectionString do WebConfig: " + DecryptConnectionString());
#endif


            connection = new SqlConnection(DecryptConnectionString());
            connection.Open();
        }
        #endregion

        #region SELECT
        protected void DoSelect(DataSet ds)
        {
            try
            {
                BeginProcessing = DateTime.Now;

                TableMapping(ds);

                selectCommand.Connection = connection;
                selectCommand.CommandType = System.Data.CommandType.StoredProcedure;
                selectCommand.CommandTimeout = TimeoutTime;
                //
                dataAdapter.SelectCommand = selectCommand;

                try
                {
                    SetInitialParameters();

                    dataAdapter.Fill(ds);
                }
                catch (Exception ex)
                {
                    throw ex;
                }
                EndProcessing = DateTime.Now;

                LogSQLCommand(selectCommand, ds);
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
        protected void DoSelect(DataSet ds, string storedProcedure)
        {
            BeginProcessing = DateTime.Now;

            TableMapping(ds.Tables[0].TableName);

            selectCommand = new SqlCommand(storedProcedure);
            selectCommand.CommandTimeout = TimeoutTime;

            DoSelect(ds);

            EndProcessing = DateTime.Now;

            LogSQLCommand(selectCommand);
        }

        protected void DoSelect(DataSet ds, string storedProcedure, string tableName)
        {
            BeginProcessing = DateTime.Now;

            dataAdapter = new SqlDataAdapter();

            TableMapping(tableName);

            if (selectCommand == null)
                selectCommand = new SqlCommand();

            selectCommand.CommandText = storedProcedure;
            selectCommand.Connection = connection;
            selectCommand.CommandType = System.Data.CommandType.StoredProcedure;
            selectCommand.CommandTimeout = TimeoutTime;

            dataAdapter.SelectCommand = selectCommand;
            //
            try
            {
                SetInitialParameters();
                dataAdapter.Fill(ds);
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
                GerarLogFileSystem("\n--------------------------------------------");
                GerarLogFileSystem("ERRO\n--------------------------------------------");
                GerarLogFileSystem("Mensagem: " + ex.Message);
                GerarLogFileSystem("Data: " + ex.Data);
                GerarLogFileSystem("XML: " + ds.GetXml());
                GerarLogFileSystem("SP: " + storedProcedure);
                GerarLogFileSystem("TableName: " + tableName);
                GerarLogFileSystem("--------------------------------------------");          
                throw (ex);

            }
            EndProcessing = DateTime.Now;
            LogSQLCommand(selectCommand, ds);
        }
        #endregion

        #region UPDATE
        protected void DoUpdate()
        {
            updateCommand.Connection = connection;
            updateCommand.CommandType = System.Data.CommandType.StoredProcedure;
            updateCommand.CommandTimeout = TimeoutTime;

            try
            {
                LogSQLCommand(updateCommand);
                updateCommand.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
        #endregion

        #region INSERT
        protected void DoInsert()
        {
            insertCommand.Connection = connection;
            insertCommand.CommandType = System.Data.CommandType.StoredProcedure;
            insertCommand.CommandTimeout = TimeoutTime;

            try
            {
                LogSQLCommand(insertCommand);
                insertCommand.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
        #endregion

        #region DELETE
        protected void DoDelete()
        {
            deleteCommand.Connection = connection;
            deleteCommand.CommandType = System.Data.CommandType.StoredProcedure;
            deleteCommand.CommandTimeout = TimeoutTime;

            try
            {
                LogSQLCommand(deleteCommand);
                deleteCommand.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
        #endregion

        #region Trilha de Auditoria
        protected void GeraTrilhaAuditoria(int IDAlterado, DataRow valoresAnteriores, DataRow valoresNovos)
        {

            DateTime dataHoraAlteracao = DateTime.Now;
            int ID_Controle = 0;
            int ID_Campo = 0;

            if (IDAlterado == 0)
            {
//Se for novo           
            }


            dsTrilhaAuditoria trilha = new dsTrilhaAuditoria();
            ListarControle(trilha, valoresAnteriores.Table.TableName);

            foreach (System.Data.DataColumn campo in valoresAnteriores.Table.Columns)
            {
                ID_Controle = int.Parse(trilha.Controle[0].ID_Controle.ToString());
                ListarCampo(trilha, ID_Controle , campo.Caption);

                // Se no existe o campo na tabela de campos, cria um novo
                if (trilha.Campo.Rows.Count == 0)
                {
                    dsTrilhaAuditoria.CampoRow novoCampo = trilha.Campo.NewCampoRow();
                    novoCampo.Descricao =  campo.Caption;
                    novoCampo.ID_Controle =  ID_Controle;
                    novoCampo.ID_Campo = 0; 
                    ID_Campo = SalvarCampo(novoCampo);  
                }
                else 
                {
                    ID_Campo = trilha.Campo[0].ID_Campo; 
                }
                ;

                if (valoresAnteriores[campo.ColumnName.ToString()].ToString() != valoresNovos[campo.ColumnName.ToString()].ToString())
                {
                    SalvarLog(IDAlterado
                            , ID_Controle 
                            , ID_Campo 
                            , valoresAnteriores[campo.ColumnName].ToString() 
                            , valoresNovos[campo.ColumnName].ToString()
                            , dataHoraAlteracao);
                };
                
                //System.Diagnostics.Trace.WriteLine("-----------------------------------");
                //System.Diagnostics.Trace.WriteLine("Tabela.....:" + campo.Table.TableName);
                //System.Diagnostics.Trace.WriteLine("Coluna.....:" + campo.Caption);
                //System.Diagnostics.Trace.WriteLine("DataType...:" + campo.DataType);
                //System.Diagnostics.Trace.WriteLine("Conteudo...:" + valoresAnteriores[campo.ColumnName.ToString()]);
                //System.Diagnostics.Trace.WriteLine("Conteudo...:" + valoresNovos[campo.ColumnName.ToString()]);

            }
        }

        protected void SalvarLog(int IDAlterado, int ID_Controle, int ID_Campo, string valorAnterior, string valorNovo, DateTime dataHoraAlteracao)
        {

            //No  necessrio... j  verificado no metodo GerarTrilha
            //if (valorAnterior != valorNovo)
            //{

                int tipoLog = 2;
                // Se  uma incluso, seta 1
                if(IDAlterado == 0)
                {
                    tipoLog = 1;
                }

                selectCommand = new SqlCommand();

                selectCommand.Parameters.Add(new SqlParameter("@ID_Controle", ID_Controle ));
                selectCommand.Parameters.Add(new SqlParameter("@ID_Campo", ID_Campo));
                selectCommand.Parameters.Add(new SqlParameter("@ID_Usuario", 1));
                selectCommand.Parameters.Add(new SqlParameter("@IDAlterado", IDAlterado));
                selectCommand.Parameters.Add(new SqlParameter("@DataHoraAlteracao", dataHoraAlteracao));
                selectCommand.Parameters.Add(new SqlParameter("@ValorAnterior", valorAnterior));
                selectCommand.Parameters.Add(new SqlParameter("@ValorNovo", valorNovo));
                selectCommand.Parameters.Add(new SqlParameter("@TipoLog", tipoLog ));

                dsTrilhaAuditoria ds = new dsTrilhaAuditoria();
                DoSelect(ds, "p_SalvarLog", "SalvarLog");

//            }
        }

        protected void ListarControle(dsTrilhaAuditoria ds, string descricao)
        {
            selectCommand = new SqlCommand();
            //
            if (descricao != "")
                selectCommand.Parameters.Add(new SqlParameter("@Descricao", descricao));
            //
            DoSelect(ds, "p_ListarControle", ds.Controle.TableName);
        }

        protected void ListarCampo(dsTrilhaAuditoria ds, int ID_Controle, string descricao)
        {
            selectCommand = new SqlCommand();
            //
            if (ID_Controle != 0)
                selectCommand.Parameters.Add(new SqlParameter("@ID_Controle", ID_Controle));
                selectCommand.Parameters.Add(new SqlParameter("@Descricao", descricao));

            //
            DoSelect(ds, "p_ListarCampo", ds.Campo.TableName);
        }

        protected int SalvarCampo(dsTrilhaAuditoria.CampoRow campo)

        {
            selectCommand = new SqlCommand();
            //
            selectCommand.Parameters.Add(new SqlParameter("@ID_Campo", campo.ID_Campo));
            selectCommand.Parameters.Add(new SqlParameter("@ID_Controle", campo.ID_Controle));
            selectCommand.Parameters.Add(new SqlParameter("@Descricao", campo.Descricao));
            //

            dsTrilhaAuditoria ds = new dsTrilhaAuditoria();
            DoSelect(ds, "p_SalvarCampo", "SalvarCampo");

            return int.Parse(ds.SalvarCampo[0].Retorno.ToString());

        }

        #endregion

        #region Table Mapping
        private void TableMapping(string tableName)
        {

            if (dataAdapter != null)
            {
                dataAdapter.TableMappings.AddRange(
                    new System.Data.Common.DataTableMapping[] {
																  new System.Data.Common.DataTableMapping("Table", tableName, new System.Data.Common.DataColumnMapping[0])});
            }
        }

        private void TableMapping(string tableName, int numeroTabela)
        {
            string sourceTable = "Table";

            if (numeroTabela > 0)
                sourceTable += numeroTabela.ToString();

            if (dataAdapter != null)
            {
                dataAdapter.TableMappings.AddRange(
                    new System.Data.Common.DataTableMapping[] {
																  new System.Data.Common.DataTableMapping(sourceTable, tableName, new System.Data.Common.DataColumnMapping[0])});
            }
        }
        private void TableMapping(DataSet ds)
        {
            for (int i = 0; i < ds.Tables.Count; i++)
            {
                //System.Diagnostics.Debug.WriteLine(ds.Tables[i].TableName);
                TableMapping(ds.Tables[i].TableName, i);
            }
        }

        #endregion

        #region Conection parameters
        private void SetInitialParameters()
        {
            SqlCommand inicial = new SqlCommand();
            inicial.CommandType = CommandType.Text;
            inicial.CommandText = "SET NOCOUNT ON; SET TRANSACTION ISOLATION LEVEL READ COMMITTED";
            inicial.Connection = connection;
            try
            {
                inicial.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
        #endregion

        #region Log Comandos SQL
        private void LogSQLCommand(SqlCommand comm)
        {
#if DEBUG
            System.Diagnostics.Trace.WriteLine("=======================================================");
            System.Diagnostics.Trace.WriteLine("SQL statement: [" + comm.CommandText + "]");

            SqlParameter param = null;
            for (int i = 0; i < comm.Parameters.Count; i++)
            {
                param = comm.Parameters[i];
                System.Diagnostics.Trace.WriteLine("Parameter: [" + param.ParameterName + "]  Value: [" + param.Value + "]  Direction: [" + param.Direction.ToString() + "]");
            }
#endif
        }
        private void LogSQLCommand(SqlCommand comm, DataSet ds)
        {
#if DEBUG
            System.Diagnostics.Trace.WriteLine("---------------------------------------------------------------------------");
            System.Diagnostics.Trace.WriteLine("Begin processing: " + BeginProcessing.ToString("dd/MM/yyyy HH:mm:ss"));
            LogSQLCommand(comm);
            //
            DataTable tabela = null;
            for (int i = 0; i < ds.Tables.Count; i++)
            {
                tabela = ds.Tables[i];
                System.Diagnostics.Trace.WriteLine("Items loaded into table " + tabela.TableName.ToUpper() + ": [" + tabela.Rows.Count + "]");
            }
            System.Diagnostics.Trace.WriteLine("---------------------------------------------------------------------------");
            System.Diagnostics.Trace.WriteLine("End Processing: " + EndProcessing.ToString("dd/MM/yyyy HH:mm:ss"));
            System.Diagnostics.Trace.WriteLine("Processing duration: " + EndProcessing.Subtract(BeginProcessing));
            System.Diagnostics.Trace.WriteLine("---------------------------------------------------------------------------");
#endif
        }
        #endregion

        #region Criptografia ConnectionString
   
        private string EncryptConnectionString(string connectionString)
        {
            Byte[] b = System.Text.ASCIIEncoding.ASCII.GetBytes(connectionString);
            string encryptedConnectionString = Convert.ToBase64String(b);
            return encryptedConnectionString;
        }

        private string DecryptConnectionString()
        {
            Byte[] b = Convert.FromBase64String(ConfigurationSettings.AppSettings["ConnectionString"]);
            string decryptedConnectionString = System.Text.ASCIIEncoding.ASCII.GetString(b);
            return decryptedConnectionString;
        }

        #endregion

        #region Log filer system

        protected void GerarLogFileSystem(string mensagem)
        {
            try
            {
                string diretorio = System.AppDomain.CurrentDomain.BaseDirectory + "\\LOG\\";
                if (!System.IO.Directory.Exists(diretorio))
                    System.IO.Directory.CreateDirectory(diretorio);

                string nomeArquivo = "SACI-{0}-{1}-{2}.log";
                nomeArquivo = string.Format(nomeArquivo, DateTime.Now.Year, DateTime.Now.Month.ToString().PadLeft(2, '0'), DateTime.Now.Day.ToString().PadLeft(2, '0'));

                string filename = diretorio + nomeArquivo;
                System.IO.StreamWriter sw = new System.IO.StreamWriter(filename, true, System.Text.Encoding.Default);
                string linha = "[{0}] {1}";
                linha = string.Format(linha, DateTime.Now.ToString("dd/MM/yyyy HH:mm:ss"), mensagem);
                sw.WriteLine(linha);
                sw.Close();
            }
            catch { }
        }

        #endregion
    }

}

